package unicrawler;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicLong;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import unicrawler.config.CrawlerConfig;
import unicrawler.crawler.CrawlerMonitor;
import unicrawler.crawler.CrawlerWorkerCounter;
import unicrawler.crawler.RobotManager;
import unicrawler.downloader.IDownloader;
import unicrawler.downloader.WebpageDownloader;
import unicrawler.frontier.DefaultFrontier;
import unicrawler.frontier.IFrontier;
import unicrawler.types.CrawlerURL;

public class CrawlerEnvironment {
	
	private Logger logger = Logger.getLogger(CrawlerEnvironment.class.getName());
	
	private CrawlerConfig config;
	private RobotManager mRobotManager;
	
	private ScheduledThreadPoolExecutor workers;
	private CrawlerWorkerCounter workCounter;
	private Thread monitor;
	
	private CrawlerURL mSeed;
	private IDownloader mDownloader;
	private IFrontier mFrontier;
	
	private long startTime;
	
	private AtomicLong crawlsRedirected = new AtomicLong(0);
	private AtomicLong crawlsInvalidResponse = new AtomicLong(0);
	private AtomicLong crawlsSkipped = new AtomicLong(0);
	
	public void start(String propertyPath) {		
		startTime = System.currentTimeMillis();
		
		logger.info("Loading crawler configuration...");
		
		try {
			// Try to load our Crawler Configuration
			this.config = new CrawlerConfig(propertyPath);
			
		} catch (Exception e) {
			// Log the error
			logger.error("Unable to load crawlers properties file!", e);
			System.exit(0);	// pssh
		}
		
		this.mRobotManager = new RobotManager(this);
		
		// Setup the downloader
		this.mDownloader = new WebpageDownloader(this);
		
		this.setupThreadPool();
		this.setupFrontier();
		
		logger.info("Starting crawl with seed URL \"" + this.mSeed.getLink() + "\"");
		
		this.mFrontier.start();
		
	}
	
	private void setupThreadPool() {
		// Initialize the Scheduled thread-pool
		this.workers = new ScheduledThreadPoolExecutor(this.config.getThreadPoolSize());
		this.workers.setKeepAliveTime(this.config.getThreadAliveTime(), TimeUnit.MILLISECONDS);
		this.workers.allowCoreThreadTimeOut(true);
		
		// Initialize our (thread-safe) worker counter
		this.workCounter = new CrawlerWorkerCounter();
		
		// Initialize & Start monitoring our Scheduled Thread-pool
		this.monitor = new Thread(new CrawlerMonitor(this));
		this.monitor.setDaemon(true);
		this.monitor.start();
	}
	
	private void setupFrontier() {
		Constructor<?> frontierConstructor;
		
		this.mSeed = new CrawlerURL(this.config.getSeedUrl(), 0);
		
		try {
			frontierConstructor = Class.forName(this.config.getFrontierClassName()).getConstructor(CrawlerEnvironment.class);
			this.mFrontier = (IFrontier)frontierConstructor.newInstance(this);
			
		} catch (NoSuchMethodException | SecurityException | ClassNotFoundException | InstantiationException | IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
			logger.error("Unable to locate frontier class \"" + this.config.getFrontierClassName() + "\" - using DefaultFrontier");
			
			// Unable to locate class ns specified in settings file
			this.mFrontier = new DefaultFrontier(this);
		}
		
		this.mFrontier.enqueue(this.mSeed);
	}
	
	public RobotManager getRobotsManager() {
		return this.mRobotManager;
	}
	
	public CrawlerConfig getConfig() {
		return this.config;
	}
	
	public IDownloader getDownloader() {
		return this.mDownloader;
	}
	
	public CrawlerURL getSeedUrls() {
		return this.mSeed;
	}
	
	public ScheduledThreadPoolExecutor getThreadpool() {
		return this.workers;
	}
	
	public CrawlerWorkerCounter getThreadpoolCounter() {
		return this.workCounter;
	}
	
	public IFrontier getFrontier() {
		return this.mFrontier;
	}
	
	public long getStartTime() {
		return this.startTime;
	}
	
	public long getRedirectedCrawlCount() {
		return this.crawlsRedirected.get();
	}
	public void incrementRedirectedCount() {
		this.crawlsRedirected.incrementAndGet();
	}
	
	public long getResponseErrorCount() {
		return this.crawlsInvalidResponse.get();
	}
	public void incrementResponseErrorCount() {
		this.crawlsInvalidResponse.incrementAndGet();
	}
	public void incrementSkippedCount() {
		this.crawlsSkipped.incrementAndGet();
	}
	public long getSkippedCount() {
		return this.crawlsSkipped.get();
	}
	
	/*
	 * Main Application entrypoint
	 */
	public static void main(String args[]) {
		if (args.length < 2) {
			System.out.println("Usage: CrawlerEnvironment [log4j.properties] [path-to-crawler.properties]");
			return;
		}
		
		// configure the log4j using our property file
		PropertyConfigurator.configure(args[0]);
		
		// Kick start the environment
		new CrawlerEnvironment().start(args[1]);
	}
}
