package unicrawler.config;

import java.util.regex.Pattern;

public class CrawlerConfig {
	
	private PropertiesFile mProperties;
	
	private String seedUrl;					// The initial seed URL for crawler to start with
	private int crawlerMaxDepth;			// Maximum link depth crawler should traverse to
	private int crawlerMaxPageCount;		// Maximum number of pages crawler should crawl overall
	private boolean crawlerSeedOnly;		// Whether the crawler should keep within the Seed URL domain
	private String crawlerFrontierClass;		// Crawler Frontier type
	private boolean crawlerUseRobotManager;	// Shall we take into consideration robots.txt?
	
	private int threadPoolSize;				// Crawler Scheduled ThreadPool size
	private int threadPoolDelay;			// Scheduler delay
	private int threadKeepAlive;			// How long should threads be kept alive ready for re-use after "dieing"
	private boolean threadDebug;
	
	private String downloaderUserAgent;		// UserAgent to be used for all downloader instances
	private boolean downloaderFollowRedirects;	// Should the downloader follow redirects?
	private String[] downloaderContentTypes;	// Allowed MIME types for downloader to download.
	private int downloaderMaxRetries;			// Max retries before downloader should give-up on page
		
	private boolean showSkippedUrls;
	private boolean showResponseErrors;
	private boolean showMaxDepthWarn;
	private boolean showSeedOnlyWarn;
	private boolean showPrevCrawledWarn;
	
	public CrawlerConfig(String propertiesPath) throws Exception {		
		
		this.mProperties = new PropertiesFile();
		if ( ! this.mProperties.loadPropertyFile(propertiesPath) ) {
			// Unable to load the properties file
			throw new Exception("Unable to load Crawler's property file!");
		}
		
		// Validate the Seed URL is within the properties file...
		this.seedUrl = this.mProperties.get("crawler.seedurl", "");
		
		if (this.seedUrl.trim().length() < 1) {
			throw new IllegalArgumentException("The properties file MUST contain a seed URL!");
		}
		if ( ! Pattern.compile("(?s)^http[s]?://.*$").matcher(this.seedUrl).matches() ) {
			// Not a valid Seed URL!
			throw new IllegalArgumentException("crawler.seedurl MUST start with either http:// or https://");
		}
		
		this.crawlerMaxDepth = this.mProperties.getInt("crawler.maxdepth", 2);
		this.crawlerMaxPageCount = this.mProperties.getInt("crawler.maxpages", -1);
		this.crawlerSeedOnly = Boolean.parseBoolean(this.mProperties.get("crawler.seedonly", "false"));
		this.crawlerFrontierClass = this.mProperties.get("crawler.frontier", "unicrawler.frontier.DefaultFrontier");
		this.crawlerUseRobotManager = Boolean.parseBoolean(this.mProperties.get("crawler.userobotsprotocol", "false"));
		
		this.threadPoolSize = this.mProperties.getInt("thread.poolsize", 30);
		this.threadKeepAlive = this.mProperties.getInt("thread.keepalive", 30000);
		this.threadPoolDelay = this.mProperties.getInt("thread.pooldelay", 200);
		this.threadDebug = Boolean.parseBoolean(this.mProperties.get("thread.debug", "true"));
		
		this.downloaderUserAgent = this.mProperties.get("downloader.useragent", "UniCrawler/0.0.1 (Java)");
		this.downloaderFollowRedirects = Boolean.parseBoolean(this.mProperties.get("downloader.followredirects", "false"));
		this.downloaderContentTypes = this.mProperties.get("downloader.contenttypes", "text/html").split(",");
		this.downloaderMaxRetries = this.mProperties.getInt("downloader.maxretries", 3);
	
		this.showSkippedUrls = Boolean.parseBoolean(this.mProperties.get("log.showskipped", "false"));
		this.showResponseErrors = Boolean.parseBoolean(this.mProperties.get("log.showresponseerrors", "false"));
		this.showMaxDepthWarn = Boolean.parseBoolean(this.mProperties.get("log.showmaxdepthwarn", "false"));
		this.showSeedOnlyWarn = Boolean.parseBoolean(this.mProperties.get("log.showseedonlywarn", "false"));
		this.showPrevCrawledWarn = Boolean.parseBoolean(this.mProperties.get("log.showprevcrawledwarn", "false"));
	}
	
	public boolean showPreviouslyCrawledWarn() {
		return this.showPrevCrawledWarn;
	}
	
	public boolean showSkipped() {
		return this.showSkippedUrls;
	}
	
	public boolean showResponseErrors() {
		return this.showResponseErrors;
	}
	
	public boolean showMaxDepthWarn() {
		return this.showMaxDepthWarn;
	}
	
	public boolean showSeedOnlyWarn() {
		return this.showSeedOnlyWarn;
	}
	
	public String getSeedUrl() {
		return this.seedUrl;
	}
	
	public int getMaxCrawlDepth() {
		return this.crawlerMaxDepth;
	}
	
	public int getMaxCrawlPageCount() {
		return this.crawlerMaxPageCount;
	}
	
	public boolean onlyCrawlSeedDomain() {
		return this.crawlerSeedOnly;
	}
	
	public String getFrontierClassName() {
		return this.crawlerFrontierClass;
	}
	
	public boolean useRobotManager() {
		return this.crawlerUseRobotManager;
	}
	
	public int getThreadPoolSize() {
		return this.threadPoolSize;
	}
	
	public int getThreadPoolDelay() {
		return this.threadPoolDelay;
	}
	
	public int getThreadAliveTime() {
		return this.threadKeepAlive;
	}
	
	public boolean getThreadDebug() {
		return this.threadDebug;
	}
	
	public String getDownloaderUseragent() {
		return this.downloaderUserAgent;
	}
	
	public int getDownloaderRetries() {
		return this.downloaderMaxRetries;
	}
	
	public boolean doesDownloaderFollowRedirects() {
		return this.downloaderFollowRedirects;
	}
	
	public String[] getDownloaderAllowedContentTypes() {
		return this.downloaderContentTypes;
	}
}
