package unicrawler.crawler;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import unicrawler.CrawlerEnvironment;
import unicrawler.downloader.RobotsDownloader;

public class RobotManager {

	/** Handles any information regarding robots.txt for domains.
	 * Synchronized to be thread-safe, as multiple thread-pools will be accessing set at any 1 time
	 */
	
	private CrawlerEnvironment mEnvironment;
	
	private final String PATH_REGEX = "^Disallow:\\s*(\\S+).*$";
	
	// Key = Domain, Value = List<> of disallowed URL's
	private Map<String, List<String>> mDomains;
	
	public RobotManager(CrawlerEnvironment _env) {
		this.mEnvironment = _env;
		
		this.mDomains = Collections.synchronizedMap(new HashMap<String, List<String>>());
	}
	
	private boolean hostListed(String url) {
		// Do we have the host listed within our collection? Does this need to be Synchronized? (????)
		return this.mDomains.containsKey(url);
	}
	
	private boolean isPathDisallowed(String host, String path) {
		// We still use Synchronized here because the List<String> within our SyncronizedMap isn't thread-safe (i think?)
		synchronized (this.mDomains) {
			List<String> paths = this.mDomains.get(host);
			
			for (String disallowed : paths) {
				if (disallowed.startsWith(path)) {
					return false;
				}
			}
			
			return true;
		}
	}
	
	public boolean determineConstraint(URL _url) {
		String host = _url.getHost();
		
		if (this.hostListed(host)) {
			// We have the host in our collection...
			return this.isPathDisallowed(host, _url.getFile());
		}
		
		// If not, try to download the Hosts file...
		String robotURL = "http://" + _url.getHost() + "/robots.txt";
		String file = RobotsDownloader.downloadRobotFile(robotURL);
		
		if (file == null) {
			// TODO: Do we insert an empty collection into our set? Or will that just consume more mem...
			return true;
		}
		
		// Parse the file
		Pattern pattern = Pattern.compile(this.PATH_REGEX, Pattern.MULTILINE);
		Matcher matcher = pattern.matcher(file);
		List<String> paths = new ArrayList<String>();
		
		while (matcher.find()) {
			if (matcher.group(1).startsWith("User-agent:")) continue;
			
			paths.add(matcher.group(1));
		}
		
		synchronized (this.mDomains) {
			this.mDomains.put(_url.getHost(), paths);
		}
		
		return this.isPathDisallowed(_url.getHost(), _url.getFile());
	}
}
