package unicrawler.frontier;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Queue;

import org.apache.log4j.Logger;

import unicrawler.CrawlerEnvironment;
import unicrawler.crawler.CrawlerWorker;
import unicrawler.types.CrawlerURL;
import unicrawler.types.Page;

/* 
 * Default frontier just keeps records of URLs to be crawled and URLs that have already -
 * been crawled in memory. Will quickly max out memory in larger crawls, further implementations 
 * can be used by implementing frontier interface and specifying within properties.
 */
public class DefaultFrontier implements IFrontier {

	/* We could use Concurrent queues? Might be something to experiment with */
	private Logger logger = Logger.getLogger(DefaultFrontier.class.getName());
	
	private CrawlerEnvironment mEnvironment;
	
	private Map<String, Page> successfulCrawls;
	private List<String> crawledUrls;
	private Queue<CrawlerURL> frontierQueue;
	
	private Thread frontierThread;
	private Object mutex = new Object();
	
	private boolean isProcessing = false;
	
	public DefaultFrontier(CrawlerEnvironment _env) {
		this.mEnvironment = _env;
		
		this.frontierQueue = new LinkedList<CrawlerURL>();
		this.successfulCrawls = new HashMap<String, Page>();
		this.crawledUrls = new ArrayList<String>();
		
		// setup the frontier thread
		this.frontierThread = new Thread(new Runnable() {
			public void run() {
				processQueue();
			}
		});
	}
	
	public void start() {
		// Check we have something inside the queue...
		if (this.getQueueSize() < 1) {
			logger.error("Queue size is 0! Scheduler thread can not start unless a seed URL is present!");
			return;
		}
		
		this.isProcessing = true;
		this.frontierThread.start();
	}
	
	private void processQueue() {
		while (isProcessing) {
			CrawlerURL url = null;
			
			synchronized (mutex) {
				url = this.frontierQueue.poll();
			}
			
			if (url != null) {
				// TODO: Check politeness policy for domains / URLs, and -
				// schedule them accordingly either using execute() for instant crawl or schedule() for delayed crawl
				this.mEnvironment.getThreadpool().execute(new CrawlerWorker(this.mEnvironment, url, this.mEnvironment.getThreadpoolCounter()));
			}
		}
	}
	
	public void stop() {
		if (this.getQueueSize() > 0) {
			logger.warn("Frontier thread has been terminated pre-maturely with items still left in queue!");
		}
		
		// End the thread safely -- thread.stop() is depreciated anyhow
		this.isProcessing = false;
	}
	
	@Override
	public void enqueue(CrawlerURL url) {
		String urlStr = url.getLink();

		synchronized (mutex) {
			if ( ! this.crawledUrls.contains(urlStr) ) {
				this.frontierQueue.add(url);
				this.crawledUrls.add(urlStr);
			} else {
				if (this.mEnvironment.getConfig().showPreviouslyCrawledWarn()) {
					logger.warn("Frontier has already discovered " + urlStr);
				}
			}
		}
	}
	
	@Override
	public void addSuccessfulCrawl(CrawlerURL _url, Page _page) {
		// Add a crawled URL to our Map obj, with a ref to our Page obj
		synchronized (mutex) {
			// TODO: do we check if domain already exists? we do check before enqueueing 
			this.successfulCrawls.put(_url.getLink(), _page);
		}
	}
	
	public int getQueueSize() {
		return this.frontierQueue.size();
	}
	
	@Override
	public int getCrawledUrlCount() {
		return this.crawledUrls.size();
	}
	
	@Override
	public int getSuccessfulCrawlCount() {
		return this.successfulCrawls.size();
	}

	@Override
	public boolean hasSeenUrl(String url) {
		return this.crawledUrls.contains(url);
	}
}
